@@ -21,7 +21,6 @@ import socket
 import select
 import nntplib
 import ftplib
-import re
 import httplib as orighttplib
 from .. import LinkCheckerError, httplib2 as httplib
 from ..dns.exception import DNSException
@@ -75,10 +74,12 @@ ExcList = ExcCacheList + ExcNoCacheList
 WARN_URL_EFFECTIVE_URL = "url-effective-url"
 WARN_URL_ERROR_GETTING_CONTENT = "url-error-getting-content"
 WARN_URL_UNICODE_DOMAIN = "url-unicode-domain"
-WARN_URL_UNNORMED = "url-unnormed"
 WARN_URL_ANCHOR_NOT_FOUND = "url-anchor-not-found"
 WARN_URL_WARNREGEX_FOUND = "url-warnregex-found"
-WARN_URL_CONTENT_TOO_LARGE = "url-content-too-large"
+WARN_URL_CONTENT_SIZE_TOO_LARGE = "url-content-too-large"
+WARN_URL_CONTENT_SIZE_ZERO = "url-content-size-zero"
+WARN_URL_CONTENT_SIZE_UNEQUAL = "url-content-size-unequal"
+WARN_URL_OBFUSCATED_IP = "url-obfuscated-ip"
 WARN_FILE_MISSING_SLASH = "file-missing-slash"
 WARN_FILE_SYSTEM_PATH = "file-system-path"
 WARN_FTP_MISSING_SLASH = "ftp-missing-slash"
@@ -89,13 +90,13 @@ WARN_HTTP_EMPTY_CONTENT = "http-empty-content"
 WARN_HTTP_COOKIE_STORE_ERROR = "http-cookie-store-error"
 WARN_HTTP_DECOMPRESS_ERROR = "http-decompress-error"
 WARN_HTTP_UNSUPPORTED_ENCODING = "http-unsupported-encoding"
+WARN_HTTP_AUTH_UNKNOWN = "http-auth-unknonwn"
 WARN_IGNORE_URL = "ignore-url"
 WARN_MAIL_NO_MX_HOST = "mail-no-mx-host"
 WARN_MAIL_UNVERIFIED_ADDRESS = "mail-unverified-address"
 WARN_MAIL_NO_CONNECTION = "mail-no-connection"
 WARN_NNTP_NO_SERVER = "nntp-no-server"
 WARN_NNTP_NO_NEWSGROUP = "nntp-no-newsgroup"
-WARN_NNTP_BUSY = "nntp-busy"
 
 # registered warnings
 Warnings = {
@@ -104,11 +105,12 @@ Warnings = {
     WARN_URL_ERROR_GETTING_CONTENT:
         _("Could not get the content of the URL."),
     WARN_URL_UNICODE_DOMAIN: _("URL uses a unicode domain."),
-    WARN_URL_UNNORMED: _("URL is not normed."),
     WARN_URL_ANCHOR_NOT_FOUND: _("URL anchor was not found."),
     WARN_URL_WARNREGEX_FOUND:
         _("The warning regular expression was found in the URL contents."),
-    WARN_URL_CONTENT_TOO_LARGE: _("The URL content is too large."),
+    WARN_URL_CONTENT_SIZE_TOO_LARGE: _("The URL content size is too large."),
+    WARN_URL_CONTENT_SIZE_ZERO: _("The URL content size is zero."),
+    WARN_URL_CONTENT_SIZE_UNEQUAL: _("The URL content size and download size are unequal."),
     WARN_FILE_MISSING_SLASH: _("The file: URL is missing a trailing slash."),
     WARN_FILE_SYSTEM_PATH:
         _("The file: path is not the same as the system specific path."),
@@ -124,6 +126,8 @@ Warnings = {
         _("An error occurred while decompressing the URL content."),
     WARN_HTTP_UNSUPPORTED_ENCODING:
         _("The URL content is encoded with an unknown encoding."),
+    WARN_HTTP_AUTH_UNKNOWN:
+        _("Unsupported HTTP authentication method."),
     WARN_IGNORE_URL: _("The URL has been ignored."),
     WARN_MAIL_NO_MX_HOST: _("The mail MX host could not be found."),
     WARN_MAIL_UNVERIFIED_ADDRESS:
@@ -132,35 +136,5 @@ Warnings = {
         _("No connection to a MX host could be established."),
     WARN_NNTP_NO_SERVER: _("No NNTP server was found."),
     WARN_NNTP_NO_NEWSGROUP: _("The NNTP newsgroup could not be found."),
-    WARN_NNTP_BUSY: _("The NNTP server was busy."),
+    WARN_URL_OBFUSCATED_IP: _("The IP is obfuscated."),
 }
-
-# file extensions we can parse recursively
-PARSE_EXTENSIONS = {
-    "html": re.compile(r'(?i)\.s?html?$'),
-    "opera": re.compile(r'/(?i)opera.adr$'), # opera bookmark file
-    "css": re.compile(r'(?i)\.css$'), # CSS stylesheet
-    "swf": re.compile(r'(?i)\.swf$'), # SWF file
-    "word": re.compile(r'(?i)\.docx?$'), # Word files
-}
-
-PARSE_MIMETYPES = (
-    "text/html",
-    "application/xhtml+xml",
-    "text/css",
-    "application/x-shockwave-flash",
-    "application/msword",
-)
-
-HTML_MIMETYPES = (
-    "text/html",
-    "application/xhtml+xml",
-)
-
-# if file extension lookup was unsuccessful, look at the content
-PARSE_CONTENTS = {
-    "html": re.compile(r'^(?i)<(!DOCTYPE html|html|head|title)'),
-    "opera" : re.compile(r'^Opera Hotlist'),
-    "text" : re.compile(r'(?i)^# LinkChecker URL list'),
-}
-
