@@ -26,8 +26,7 @@ import urllib2
 
 from . import urlbase, get_index_html, absolute_url, get_url_from
 from .. import log, LOG_CHECK, fileutil, strformat, url as urlutil
-from .const import WARN_FILE_MISSING_SLASH, WARN_FILE_SYSTEM_PATH, \
-    PARSE_EXTENSIONS, PARSE_CONTENTS
+from .const import WARN_FILE_MISSING_SLASH, WARN_FILE_SYSTEM_PATH
 
 try:
     import sqlite3
@@ -35,6 +34,10 @@ try:
 except ImportError:
     has_sqlite = False
 
+
+firefox_extension = re.compile(r'/(?i)places.sqlite$')
+
+
 def get_files (dirname):
     """Get iterator of entries in directory. Only allows regular files
     and directories, no symlinks."""
@@ -77,12 +80,10 @@ def is_absolute_path (path):
     with a drive letter. On all other systems absolute paths start with
     a slash."""
     if os.name == 'nt':
-       return re.search(r"^[a-zA-Z]:", path)
+        return re.search(r"^[a-zA-Z]:", path)
     return path.startswith("/")
 
 
-firefox_extension = re.compile(r'/(?i)places.sqlite$')
-
 class FileUrl (urlbase.UrlBase):
     """
     Url link with file scheme.
@@ -112,6 +113,8 @@ class FileUrl (urlbase.UrlBase):
             base_url = base_url.replace("\\", "/")
             # transform c:/windows into /c|/windows
             base_url = re.sub("^file://(/?)([a-zA-Z]):", r"file:///\2|", base_url)
+            # transform file://path into file:///path
+            base_url = re.sub("^file://([^/])", r"file:///\1", base_url)
         self.base_url = unicode(base_url)
 
     def build_url (self):
@@ -127,6 +130,16 @@ class FileUrl (urlbase.UrlBase):
             self.urlparts[2] += '/'
         self.url = urlparse.urlunsplit(self.urlparts)
 
+    def add_size_info (self):
+        """Get size of file content from filename path."""
+        if self.is_directory():
+            # Directory size always differs from the customer index.html
+            # that is generated. So return without calculating any size.
+            return
+        self.size = fileutil.get_size(self.get_os_filename())
+        if self.dlsize == -1:
+            self.dlsize = self.size
+
     def check_connection (self):
         """
         Try to open the local file. Under NT systems the case sensitivity
@@ -163,25 +176,22 @@ class FileUrl (urlbase.UrlBase):
             data = get_index_html(get_files(self.get_os_filename()))
             if isinstance(data, unicode):
                 data = data.encode("iso8859-1", "ignore")
+            size = len(data)
         else:
-            data = super(FileUrl, self).read_content()
-        return data
+            data, size = super(FileUrl, self).read_content()
+        return data, size
 
     def is_html (self):
-        """
-        Check if file is a HTML file.
-        """
-        if PARSE_EXTENSIONS['html'].search(self.url):
-            return True
-        if PARSE_CONTENTS['html'].search(self.get_content()):
-            return True
-        return False
+        """Check if file is a HTML file."""
+        mime = fileutil.guess_mimetype(self.url, read=self.get_content)
+        return self.ContentMimetypes.get(mime) == "html"
 
     def is_css (self):
         """
         Check if file is a CSS file.
         """
-        return bool(PARSE_EXTENSIONS['css'].search(self.url))
+        mime = fileutil.guess_mimetype(self.url, read=self.get_content)
+        return self.ContentMimetypes.get(mime) == "css"
 
     def is_file (self):
         """
@@ -227,20 +237,11 @@ class FileUrl (urlbase.UrlBase):
         """
         if self.is_directory():
             return True
-        # guess by extension
-        for ro in PARSE_EXTENSIONS.values():
-            if ro.search(self.url):
-                return True
-        if firefox_extension.search(self.url):
+        elif has_sqlite and firefox_extension.search(self.url):
             return True
-        # try to read content (can fail, so catch error)
-        try:
-            for ro in PARSE_CONTENTS.values():
-                if ro.search(self.get_content()[:30]):
-                    return True
-        except IOError:
-            pass
-        return False
+        else:
+            mime = fileutil.guess_mimetype(self.url, read=self.get_content)
+            return mime in self.ContentMimetypes
 
     def parse_url (self):
         """
@@ -248,18 +249,12 @@ class FileUrl (urlbase.UrlBase):
         """
         if self.is_directory():
             self.parse_html()
-            return
-        for key, ro in PARSE_EXTENSIONS.items():
-            if ro.search(self.url):
-                getattr(self, "parse_"+key)()
-                return
-        if has_sqlite and firefox_extension.search(self.url):
+        elif has_sqlite and firefox_extension.search(self.url):
             self.parse_firefox()
-            return
-        for key, ro in PARSE_CONTENTS.items():
-            if ro.search(self.get_content()[:30]):
-                getattr(self, "parse_"+key)()
-                return
+        else:
+            mime = fileutil.guess_mimetype(self.url, read=self.get_content)
+            key = self.ContentMimetypes[mime]
+            getattr(self, "parse_"+key)()
 
     def parse_firefox (self):
         """Parse a Firefox3 bookmark file."""
