@@ -35,7 +35,7 @@ from .const import WARN_HTTP_ROBOTS_DENIED, \
     WARN_HTTP_WRONG_REDIRECT, WARN_HTTP_MOVED_PERMANENT, \
     WARN_HTTP_EMPTY_CONTENT, WARN_HTTP_COOKIE_STORE_ERROR, \
     WARN_HTTP_DECOMPRESS_ERROR, WARN_HTTP_UNSUPPORTED_ENCODING, \
-    PARSE_MIMETYPES, HTML_MIMETYPES
+    WARN_HTTP_AUTH_UNKNOWN
 
 # helper alias
 unicode_safe = strformat.unicode_safe
@@ -48,55 +48,6 @@ _supported_encodings = ('gzip', 'x-gzip', 'deflate')
 # Amazon blocks all HEAD requests
 _is_amazon = re.compile(r'^www\.amazon\.(com|de|ca|fr|co\.(uk|jp))').search
 
-# Stolen from Python CVS urllib2.py
-# Mapping status codes to official W3C names
-httpresponses = {
-    100: 'Continue',
-    101: 'Switching Protocols',
-
-    200: 'OK',
-    201: 'Created',
-    202: 'Accepted',
-    203: 'Non-Authoritative Information',
-    204: 'No Content',
-    205: 'Reset Content',
-    206: 'Partial Content',
-
-    300: 'Multiple Choices',
-    301: 'Moved Permanently',
-    302: 'Found',
-    303: 'See Other',
-    304: 'Not Modified',
-    305: 'Use Proxy',
-    306: '(Unused)',
-    307: 'Temporary Redirect',
-
-    400: 'Bad Request',
-    401: 'Unauthorized',
-    402: 'Payment Required',
-    403: 'Forbidden',
-    404: 'Not Found',
-    405: 'Method Not Allowed',
-    406: 'Not Acceptable',
-    407: 'Proxy Authentication Required',
-    408: 'Request Timeout',
-    409: 'Conflict',
-    410: 'Gone',
-    411: 'Length Required',
-    412: 'Precondition Failed',
-    413: 'Request Entity Too Large',
-    414: 'Request-URI Too Long',
-    415: 'Unsupported Media Type',
-    416: 'Requested Range Not Satisfiable',
-    417: 'Expectation Failed',
-
-    500: 'Internal Server Error',
-    501: 'Not Implemented',
-    502: 'Bad Gateway',
-    503: 'Service Unavailable',
-    504: 'Gateway Timeout',
-    505: 'HTTP Version Not Supported',
-}
 
 class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
     """
@@ -124,11 +75,14 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         self._data = None
         # flag indicating connection reuse
         self.reused_connection = False
+        # flag telling if GET method is allowed; determined by robots.txt
+        self.method_get_allowed = True
 
     def allows_robots (self, url):
         """
         Fetch and parse the robots.txt of given url. Checks if LinkChecker
-        can access the requested resource.
+        can get the requested resource content. HEAD requests however are
+        still allowed.
 
         @param url: the url to be requested
         @type url: string
@@ -142,6 +96,21 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         return rb.allows_url(roboturl, url, self.proxy, user, password,
             callback=callback)
 
+    def add_size_info (self):
+        """Get size of URL content from HTTP header."""
+        if self.headers and "Content-Length" in self.headers and \
+           "Transfer-Encoding" not in self.headers:
+            # Note that content-encoding causes size differences since
+            # the content data is always decoded.
+            try:
+                self.size = int(self.headers["Content-Length"])
+                if self.dlsize == -1:
+                    self.dlsize = self.size
+            except (ValueError, OverflowError):
+                pass
+        else:
+            self.size = -1
+
     def check_connection (self):
         """
         Check a URL with HTTP protocol.
@@ -165,18 +134,17 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if not self.allows_robots(self.url):
             # remove all previously stored results
             self.add_warning(
-                       _("Access denied by robots.txt, checked only syntax."),
-                       tag=WARN_HTTP_ROBOTS_DENIED)
-            self.set_result(u"syntax OK")
-            return
+                 _("Access denied by robots.txt, skipping content checks."),
+                 tag=WARN_HTTP_ROBOTS_DENIED)
+            self.method_get_allowed = False
+        # first try with HEAD
+        self.method = "HEAD"
         # check for amazon server quirk
         if _is_amazon(self.urlparts[1]):
-            self.add_info(_("Amazon servers block HTTP HEAD requests, "
-                            "using GET instead."))
-            self.method = "GET"
-        else:
-            # first try with HEAD
-            self.method = "HEAD"
+            self.add_info(_("Amazon servers block HTTP HEAD requests."))
+            if self.method_get_allowed:
+                self.add_info(_("Using GET method for Amazon server."))
+                self.method = "GET"
         # check the http connection
         response = self.check_http_connection()
         if self.headers and "Server" in self.headers:
@@ -188,13 +156,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                             "a GET request was used instead.") %
                             {"name": server})
         # redirections might have changed the URL
-        newurl = urlparse.urlunsplit(self.urlparts)
-        if self.url != newurl:
-            if self.warn_redirect:
-                log.warn(LOG_CHECK, _("""URL `%(url)s' has been redirected.
-Use URL `%(newurl)s' instead for checking.""") % {
-                'url': self.url, 'newurl': newurl})
-            self.url = newurl
+        self.url = urlparse.urlunsplit(self.urlparts)
         # check response
         if response:
             self.check_response(response)
@@ -214,10 +176,10 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 response.close()
             try:
                 response = self._try_http_response()
-            except httplib.BadStatusLine:
+            except httplib.BadStatusLine, msg:
                 # some servers send empty HEAD replies
-                if self.method == "HEAD":
-                    log.debug(LOG_CHECK, "Empty status line: falling back to GET")
+                if self.method == "HEAD" and self.method_get_allowed:
+                    log.debug(LOG_CHECK, "Bad status line %r: falling back to GET", msg)
                     self.method = "GET"
                     self.aliases = []
                     self.fallback_get = True
@@ -247,10 +209,10 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 self.proxy, self.proxyauth = oldproxy
             try:
                 tries, response = self.follow_redirections(response)
-            except httplib.BadStatusLine:
+            except httplib.BadStatusLine, msg:
                 # some servers send empty HEAD replies
-                if self.method == "HEAD":
-                    log.debug(LOG_CHECK, "Empty status line: falling back to GET")
+                if self.method == "HEAD" and self.method_get_allowed:
+                    log.debug(LOG_CHECK, "Bad status line %r: falling back to GET", msg)
                     self.method = "GET"
                     self.aliases = []
                     self.fallback_get = True
@@ -259,9 +221,10 @@ Use URL `%(newurl)s' instead for checking.""") % {
             if tries == -1:
                 log.debug(LOG_CHECK, "already handled")
                 response.close()
+                self.do_check_content = False
                 return None
             if tries >= self.max_redirects:
-                if self.method == "HEAD":
+                if self.method == "HEAD" and self.method_get_allowed:
                     # Microsoft servers tend to recurse HEAD requests
                     self.method = "GET"
                     self.aliases = []
@@ -272,6 +235,15 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 return response
             # user authentication
             if response.status == 401:
+                authenticate = self.headers.get('WWW-Authenticate')
+                if not authenticate or not authenticate.startswith("Basic"):
+                    # LinkChecker only supports Basic authorization
+                    args = {"auth": authenticate}
+                    self.add_warning(
+                       _("Unsupported HTTP authentication `%(auth)s', " \
+                         "only `Basic' authentication is supported.") % args,
+                       tag=WARN_HTTP_AUTH_UNKNOWN)
+                    return
                 if not self.auth:
                     import base64
                     _user, _password = self.get_user_password()
@@ -282,11 +254,11 @@ Use URL `%(newurl)s' instead for checking.""") % {
                     continue
             elif response.status >= 400:
                 # retry with GET (but do not set fallback flag)
-                if self.method == "HEAD":
+                if self.method == "HEAD" and self.method_get_allowed:
                     self.method = "GET"
                     self.aliases = []
                     continue
-            elif self.headers and self.method == "HEAD":
+            elif self.headers and self.method == "HEAD" and self.method_get_allowed:
                 # test for HEAD support
                 mime = headers.get_content_type(self.headers)
                 poweredby = self.headers.get('X-Powered-By', '')
@@ -323,16 +295,17 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 pass # XXX warn about idn use
             log.debug(LOG_CHECK, "Norm redirected to %r", redirected)
             urlparts = strformat.url_unicode_split(redirected)
-            # check extern filter again
-            self.set_extern(redirected)
-            if self.extern[0] and self.extern[0]:
-                if set_result:
-                    self.check301status(response)
-                    self.add_info(
-                          _("The redirected URL is outside of the domain "
-                            "filter, checked only syntax."))
-                    self.set_result(u"filtered")
-                return -1, response
+            if urlparts[1] != self.urlparts[1]:
+                # check extern filter again
+                self.set_extern(redirected)
+                if self.extern[0] and self.extern[1]:
+                    if set_result:
+                        self.check301status(response)
+                        self.add_info(
+                             _("The redirected URL is outside of the domain "
+                               "filter, checked only syntax."))
+                        self.set_result(u"filtered")
+                    return -1, response
             # check robots.txt allowance again
             if not self.allows_robots(redirected):
                 if set_result:
@@ -345,7 +318,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
             # see about recursive redirect
             all_seen = [self.cache_url_key] + self.aliases
             if redirected in all_seen:
-                if self.method == "HEAD":
+                if self.method == "HEAD" and self.method_get_allowed:
                     # Microsoft servers tend to recurse HEAD requests
                     # fall back to the original url and use GET
                     return self.max_redirects, response
@@ -355,9 +328,26 @@ Use URL `%(newurl)s' instead for checking.""") % {
                           _("recursive redirection encountered:\n %(urls)s") %
                             {"urls": "\n  => ".join(recursion)}, valid=False)
                 return -1, response
-            if urlparts[0] == self.scheme:
-                # remember redireced url as alias
+            if urlparts[0] == self.scheme or urlparts[0] in ('http', 'https'):
+                # remember redirected url as alias
                 self.aliases.append(redirected)
+            else:
+                # in case of changed scheme make new URL object
+                newobj = get_url_from(
+                          redirected, self.recursion_level, self.aggregate,
+                          parent_url=self.parent_url, base_ref=self.base_ref,
+                          line=self.line, column=self.column, name=self.name)
+                if set_result:
+                    self.add_warning(
+                     _("Redirection to URL `%(newurl)s' with different scheme"
+                       " found; the original URL was `%(url)s'.") %
+                     {"url": self.url, "newurl": newobj.url},
+                     tag=WARN_HTTP_WRONG_REDIRECT)
+                    self.set_result(u"syntax OK")
+                # append new object to queue
+                self.aggregate.urlqueue.put(newobj)
+                # pretend to be finished and logged
+                return -1, response
             # note: urlparts has to be a list
             self.urlparts = urlparts
             if set_result:
@@ -365,22 +355,6 @@ Use URL `%(newurl)s' instead for checking.""") % {
             # check cache again on the changed URL
             if self.aggregate.urlqueue.checked_redirect(redirected, self):
                 return -1, response
-            # in case of changed scheme make new URL object
-            if self.urlparts[0] != self.scheme:
-                if set_result:
-                    self.add_warning(
-                           _("Redirection to different URL type encountered; "
-                             "the original URL was `%(url)s'.") %
-                             {"url": self.url},
-                           tag=WARN_HTTP_WRONG_REDIRECT)
-                newobj = get_url_from(
-                          redirected, self.recursion_level, self.aggregate,
-                          parent_url=self.parent_url, base_ref=self.base_ref,
-                          line=self.line, column=self.column, name=self.name)
-                # append new object to queue
-                self.aggregate.urlqueue.put(newobj)
-                # pretend to be finished and logged
-                return -1, response
             # new response data
             response.close()
             response = self._try_http_response()
@@ -419,15 +393,13 @@ Use URL `%(newurl)s' instead for checking.""") % {
             # store cookies for valid links
             if self.aggregate.config['storecookies']:
                 for c in self.cookies:
-                    self.add_info(_("Store cookie: %(cookie)s.") %
+                    self.add_info(_("Sent cookie: %(cookie)s.") %
                                   {"cookie": c})
                 try:
                     out = self.aggregate.cookies.add(self.headers,
                                                      self.urlparts[0],
                                                      self.urlparts[1],
                                                      self.urlparts[2])
-                    for h in out:
-                        self.add_info(unicode_safe(h))
                 except Cookie.CookieError, msg:
                     self.add_warning(_("Could not store cookies: %(msg)s.") %
                                      {'msg': str(msg)},
@@ -455,7 +427,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 return self._get_http_response()
             raise
         except httplib.BadStatusLine, msg:
-            if str(msg) == "Empty status line" and self.reused_connection:
+            if not msg and self.reused_connection:
                 # server closed connection - retry
                 log.debug(LOG_CHECK, "Empty status line: retry")
                 self.persistent = False
@@ -487,7 +459,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
                                         self.urlparts[3], anchor))
         self.url_connection.putrequest(self.method, path, skip_host=True,
                                        skip_accept_encoding=True)
-        self.url_connection.putheader("Host", host)
+        # be sure to use the original host as header even for proxies
+        self.url_connection.putheader("Host", self.urlparts[1])
         # userinfo is from http://user@pass:host/
         if self.userinfo:
             self.url_connection.putheader("Authorization", self.userinfo)
@@ -515,7 +488,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
                 value = c.client_header_value()
                 self.url_connection.putheader(name, value)
         self.url_connection.endheaders()
-        response = self.url_connection.getresponse()
+        response = self.url_connection.getresponse(True)
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
         self.persistent = not response.will_close
@@ -532,8 +505,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
             self._read_content(response)
             assert not response.will_close
         # If possible, use official W3C HTTP response name
-        if response.status in httpresponses:
-            response.reason = httpresponses[response.status]
+        if response.status in httplib.responses:
+            response.reason = httplib.responses[response.status]
         return response
 
     def get_http_object (self, host, scheme):
@@ -574,18 +547,26 @@ Use URL `%(newurl)s' instead for checking.""") % {
         @return: URL content, decompressed and decoded
         @rtype: string
         """
+        assert self.method_get_allowed, 'unallowed content read'
         self.method = "GET"
         response = self._try_http_response()
         response = self.follow_redirections(response, set_result=False)[1]
         self.headers = response.msg
+        # Re-read size info, since the GET request result could be different
+        # than a former HEAD request.
+        self.add_size_info()
         if self._data is None:
             self._read_content(response)
-        data = self._data
-        self._data = None
-        return data
+        data, size = self._data, self._size
+        self._data = self._size = None
+        return data, size
 
     def _read_content (self, response):
+        """Read URL contents and store then in self._data.
+        This way, the method can be called by other functions than
+        read_content()"""
         data = response.read()
+        self._size = len(data)
         encoding = headers.get_content_encoding(self.headers)
         if encoding in _supported_encodings:
             try:
@@ -616,6 +597,27 @@ Use URL `%(newurl)s' instead for checking.""") % {
             return False
         return True
 
+    def set_title_from_content (self):
+        """Check if it's allowed to read content before execution."""
+        if self.method_get_allowed:
+            super(HttpUrl, self).set_title_from_content()
+
+    def get_anchors (self):
+        """Check if it's allowed to read content before execution."""
+        if self.method_get_allowed:
+            super(HttpUrl, self).get_anchors()
+
+    def content_allows_robots (self):
+        """Check if it's allowed to read content before execution."""
+        if not self.method_get_allowed:
+            return False
+        return super(HttpUrl, self).content_allows_robots()
+
+    def check_warningregex (self):
+        """Check if it's allowed to read content before execution."""
+        if self.method_get_allowed:
+            super(HttpUrl, self).check_warningregex()
+
     def is_html (self):
         """
         See if this URL points to a HTML file by looking at the
@@ -626,7 +628,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
         """
         if not (self.valid and self.headers):
             return False
-        if headers.get_content_type(self.headers) not in HTML_MIMETYPES:
+        mime = headers.get_content_type(self.headers)
+        if self.ContentMimetypes.get(mime) != "html":
             return False
         return self.encoding_supported()
 
@@ -634,7 +637,8 @@ Use URL `%(newurl)s' instead for checking.""") % {
         """Return True iff content of this url is CSS stylesheet."""
         if not (self.valid and self.headers):
             return False
-        if headers.get_content_type(self.headers) != "text/css":
+        mime = headers.get_content_type(self.headers)
+        if self.ContentMimetypes.get(mime) != "css":
             return False
         return self.encoding_supported()
 
@@ -656,7 +660,7 @@ Use URL `%(newurl)s' instead for checking.""") % {
         """
         if not (self.valid and self.headers):
             return False
-        if headers.get_content_type(self.headers) not in PARSE_MIMETYPES:
+        if headers.get_content_type(self.headers) not in self.ContentMimetypes:
             return False
         return self.encoding_supported()
 
