@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2005-2009 Bastian Kleineidam
+# Copyright (C) 2005-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,6 +52,8 @@ class InternPatternUrl (urlbase.UrlBase):
         args = list(re.escape(x) for x in (scheme, domain, path))
         if args[0] in ('http', 'https'):
             args[0] = 'https?'
+        # allow redirection www.example.com -> example.com and vice versa
         if args[1].startswith('www\\.'):
-            args[1] = r"(www\.|)%s" % args[1][5:]
+            args[1] = args[1][5:]
+        args[1] = r"(www\.|)%s" % args[1]
         return "%s://%s%s" % tuple(args)
