@@ -30,16 +30,16 @@ import select
 import tempfile
 
 from . import absolute_url, StoringHandler, get_url_from
-from ..cache import geoip
 from .. import (log, LOG_CHECK, LOG_CACHE, httputil, httplib2 as httplib,
-    strformat, LinkCheckerError, url as urlutil, trace, clamav, containers,
-    winutil)
+  strformat, LinkCheckerError, url as urlutil, trace, clamav, winutil, geoip)
 from ..HtmlParser import htmlsax
 from ..htmlutil import linkparse
+from ..network import iputil
 from .const import (WARN_URL_EFFECTIVE_URL, WARN_URL_UNICODE_DOMAIN,
-    WARN_URL_UNNORMED, WARN_URL_ERROR_GETTING_CONTENT,
+    WARN_URL_ERROR_GETTING_CONTENT, WARN_URL_OBFUSCATED_IP,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
-    WARN_URL_CONTENT_TOO_LARGE, ExcList, ExcSyntaxList, ExcNoCacheList)
+    WARN_URL_CONTENT_SIZE_TOO_LARGE, WARN_URL_CONTENT_SIZE_ZERO,
+    WARN_URL_CONTENT_SIZE_UNEQUAL, ExcList, ExcSyntaxList, ExcNoCacheList)
 
 # helper alias
 unicode_safe = strformat.unicode_safe
@@ -52,6 +52,10 @@ def urljoin (parent, url, scheme):
     """
     if url.startswith(scheme+":"):
         return url
+    # work around a Python 2.6/3.1 bug cutting off characters when the URL
+    # begins with semicolon
+    if url.startswith(';'):
+        url = "./%s" % url
     return urlparse.urljoin(parent, url)
 
 
@@ -69,6 +73,17 @@ def url_norm (url, encoding=None):
 class UrlBase (object):
     """An URL with additional information like validity etc."""
 
+    # file types that can be parsed recursively
+    ContentMimetypes = {
+        "text/html": "html",
+        "application/xhtml+xml": "html",
+        "text/css": "css",
+        "application/x-shockwave-flash": "swf",
+        "application/msword": "word",
+        "text/plain+linkchecker": "text",
+        "text/plain+opera": "opera",
+    }
+
     def __init__ (self, base_url, recursion_level, aggregate,
                   parent_url=None, base_ref=None, line=-1, column=-1,
                   name=u"", url_encoding=None):
@@ -90,7 +105,6 @@ class UrlBase (object):
         self.reset()
         self.check_syntax()
 
-
     def init (self, base_ref, base_url, parent_url, recursion_level,
               aggregate, line, column, name, url_encoding):
         """
@@ -115,8 +129,6 @@ class UrlBase (object):
         url = absolute_url(base_url, base_ref, parent_url)
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0] or "file"
-        # warn if URL is redirected (for commandline client)
-        self.warn_redirect = False
 
     def reset (self):
         """
@@ -144,6 +156,8 @@ class UrlBase (object):
         self.warnings = []
         # list of infos
         self.info = []
+        # content size
+        self.size = -1
         # download time
         self.dltime = -1
         # download size
@@ -163,6 +177,8 @@ class UrlBase (object):
         self.caching = True
         # title is either the URL or parsed from content
         self.title = None
+        # flag if content should be checked or not
+        self.do_check_content = True
 
     def set_result (self, msg, valid=True, overwrite=False):
         """
@@ -198,9 +214,9 @@ class UrlBase (object):
 
     def set_title_from_content (self):
         """Set title of page the URL refers to.from page content."""
-        if self.valid and self.is_html():
+        if self.valid:
             try:
-                handler = linkparse.TitleFinder(self.get_content())
+                handler = linkparse.TitleFinder()
             except tuple(ExcList):
                 return
             parser = htmlsax.parser(handler)
@@ -354,10 +370,6 @@ class UrlBase (object):
                           the URL %(idna_url)r instead.""") % \
                           {"url": self.base_url, "idna_url": base_url},
                           tag=WARN_URL_UNICODE_DOMAIN)
-        elif self.base_url != base_url:
-            self.add_warning(
-              _("Base URL is not properly normed. Normed URL is %(url)s.") %
-               {'url': base_url}, tag=WARN_URL_UNNORMED)
         # make url absolute
         if self.base_ref:
             # use base reference as parent url
@@ -398,6 +410,21 @@ class UrlBase (object):
                 raise LinkCheckerError(_("URL has invalid port %(port)r") %
                     {"port": str(self.port)})
             self.port = int(self.port)
+        self.check_obfuscated_ip()
+
+    def check_obfuscated_ip (self):
+        """Warn if host of this URL is obfuscated IP address."""
+        # check if self.host can be an IP address
+        if self.scheme not in ("ftp", "http", "mailto", "news", "nntp", "telnet"):
+            return
+        # check for obfuscated IP address
+        if iputil.is_obfuscated_ip(self.host):
+             ips = iputil.resolve_host(self.host)
+             if ips:
+                self.add_warning(
+                   _("URL %(url)s has obfuscated IP address %(ip)s") % \
+                   {"url": self.base_url, "ip": ips.pop()},
+                          tag=WARN_URL_OBFUSCATED_IP)
 
     def check (self):
         """Main check function for checking this URL."""
@@ -418,14 +445,18 @@ class UrlBase (object):
             self.close_connection()
 
     def add_country_info (self):
-        """
-        Try to ask GeoIP database for country info.
-        """
-        country = geoip.get_country(self.host)
-        if country is not None:
-            self.add_info(_("URL is located in %(country)s.") %
+        """Try to ask GeoIP database for country info."""
+        if self.host:
+            country = geoip.get_country(self.host)
+            if country is not None:
+                self.add_info(_("URL is located in %(country)s.") %
                 {"country": _(country)})
 
+    def add_size_info (self):
+        """Store size of URL content from meta info into self.size.
+        Must be implemented in subclasses."""
+        pass
+
     def local_check (self):
         """Local check function can be overridden in subclasses."""
         log.debug(LOG_CHECK, "Checking %s", self)
@@ -440,8 +471,8 @@ class UrlBase (object):
         log.debug(LOG_CHECK, "checking connection")
         try:
             self.check_connection()
+            self.add_size_info()
             self.add_country_info()
-            self.check_content()
         except tuple(ExcList):
             value = self.handle_exception()
             # make nicer error msg for unknown hosts
@@ -452,16 +483,18 @@ class UrlBase (object):
                 value = _('Bad HTTP response %(line)r') % {"line": str(value)}
             self.set_result(unicode_safe(value), valid=False)
         self.checktime = time.time() - check_start
-        # check recursion
-        try:
-            if self.allows_recursion():
-                self.parse_url()
-            # check content size
-            self.check_size()
-        except tuple(ExcList):
-            value = self.handle_exception()
-            self.add_warning(_("could not get content: %(msg)r") %
-                 {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
+        if self.do_check_content:
+            # check content and recursion
+            try:
+                self.check_content()
+                if self.allows_recursion():
+                    self.parse_url()
+                # check content size
+                self.check_size()
+            except tuple(ExcList):
+                value = self.handle_exception()
+                self.add_warning(_("could not get content: %(msg)r") %
+                     {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
 
     def close_connection (self):
         """
@@ -519,7 +552,7 @@ class UrlBase (object):
             log.debug(LOG_CHECK, "... no, cannot get content.")
             return False
         rec_level = self.aggregate.config["recursionlevel"]
-        if  rec_level >= 0 and self.recursion_level >= rec_level:
+        if rec_level >= 0 and self.recursion_level >= rec_level:
             log.debug(LOG_CHECK, "... no, maximum recursion level reached.")
             return False
         if self.extern[0]:
@@ -533,7 +566,7 @@ class UrlBase (object):
 
     def content_allows_robots (self):
         """
-        Return True if the content of this URL forbids robots to
+        Return False if the content of this URL forbids robots to
         search for recursive links.
         """
         if not self.is_html():
@@ -559,7 +592,7 @@ class UrlBase (object):
         """Store list of anchors for this URL. Precondition: this URL is
         an HTML resource."""
         log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
-        handler = linkparse.LinkFinder(self.get_content(), self.add_anchor,
+        handler = linkparse.LinkFinder(self.add_anchor,
                                    tags={'a': [u'name'], None: [u'id']})
         parser = htmlsax.parser(handler)
         handler.parser = parser
@@ -627,14 +660,15 @@ class UrlBase (object):
         if self.data is None:
             log.debug(LOG_CHECK, "Get content of %r", self.url)
             t = time.time()
-            self.data = self.read_content()
+            self.data, self.dlsize = self.read_content()
             self.dltime = time.time() - t
-            self.dlsize = len(self.data)
         return self.data
 
     def read_content (self):
-        """Return data for this URL. Can be overridden in subclasses."""
-        return self.url_connection.read()
+        """Return data and data size for this URL.
+        Can be overridden in subclasses."""
+        data = self.url_connection.read()
+        return data, len(data)
 
     def check_content (self):
         """Check content data for warnings, syntax errors, viruses etc."""
@@ -646,17 +680,7 @@ class UrlBase (object):
                 self.get_anchors()
         if self.anchor:
             self.check_anchor()
-        warningregex = self.aggregate.config["warningregex"]
-        if warningregex:
-            log.debug(LOG_CHECK, "checking content")
-            try:
-                match = warningregex.search(self.get_content())
-                if match:
-                    self.add_warning(_("Found %(match)r in link contents.") %
-                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
-            except tuple(ExcList):
-                value = self.handle_exception()
-                self.set_result(unicode_safe(value), valid=False)
+        self.check_warningregex()
         # is it an intern URL?
         if not self.extern[0]:
             # check HTML/CSS syntax
@@ -672,18 +696,40 @@ class UrlBase (object):
             if self.aggregate.config["scanvirus"]:
                 self.scan_virus()
 
+    def check_warningregex (self):
+        warningregex = self.aggregate.config["warningregex"]
+        if warningregex:
+            log.debug(LOG_CHECK, "checking content")
+            try:
+                match = warningregex.search(self.get_content())
+                if match:
+                    self.add_warning(_("Found %(match)r in link contents.") %
+                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
+            except tuple(ExcList):
+                value = self.handle_exception()
+                self.set_result(unicode_safe(value), valid=False)
+
     def check_size (self):
+        """Check content size if it is zero or larger than a given
+        maximum size.
         """
-        If a maximum size was given, call this function to check it
-        against the content size of this url.
-        """
-        maxbytes = self.aggregate.config["warnsizebytes"]
-        if maxbytes is not None and self.dlsize >= maxbytes:
-            self.add_warning(
+        if self.dlsize == 0:
+            self.add_warning(_("Content size is zero."),
+                             tag=WARN_URL_CONTENT_SIZE_ZERO)
+        else:
+            maxbytes = self.aggregate.config["warnsizebytes"]
+            if maxbytes is not None and self.dlsize >= maxbytes:
+                self.add_warning(
                    _("Content size %(dlsize)s is larger than %(maxbytes)s.") %
                         {"dlsize": strformat.strsize(self.dlsize),
                          "maxbytes": strformat.strsize(maxbytes)},
-                          tag=WARN_URL_CONTENT_TOO_LARGE)
+                          tag=WARN_URL_CONTENT_SIZE_TOO_LARGE)
+        if self.size != -1 and self.dlsize != -1 and self.dlsize != self.size:
+                self.add_warning(_("Download size (%(dlsize)d Byte) "
+                        "does not equal content size (%(size)d Byte).") %
+                        {"dlsize": self.dlsize,
+                         "size": self.size},
+                          tag=WARN_URL_CONTENT_SIZE_UNEQUAL)
 
     def check_html (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
@@ -794,7 +840,7 @@ class UrlBase (object):
             h.putheader('Content-Length', str(len(body)))
             h.endheaders()
             h.send(body)
-            r = h.getresponse()
+            r = h.getresponse(True)
             if r.getheader('X-W3C-Validator-Status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid CSS syntax"))
                 return
@@ -827,14 +873,10 @@ class UrlBase (object):
         self.parse_html()
 
     def get_user_password (self):
+        """Get tuple (user, password) from configured authentication.
+        Both user and password can be None.
         """
-        Get tuple (user, password) from configured authentication.
-        Both user and password can be None if not specified.
-        """
-        for auth in self.aggregate.config["authentication"]:
-            if auth['pattern'].match(self.url):
-                return auth['user'], auth['password']
-        return None, None
+        return self.aggregate.config.get_user_password(self.url)
 
     def parse_html (self):
         """Parse into HTML content and search for URLs to check.
@@ -842,7 +884,7 @@ class UrlBase (object):
         """
         log.debug(LOG_CHECK, "Parsing HTML %s", self)
         # construct parser object
-        handler = linkparse.LinkFinder(self.get_content(), self.add_url)
+        handler = linkparse.LinkFinder(self.add_url)
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
@@ -1011,7 +1053,7 @@ class UrlBase (object):
         """
         return u"<%s >" % self.serialized()
 
-    def to_wire (self):
+    def to_wire_dict (self):
         """Return a simplified transport object for logging.
 
         The transport object must contain these attributes:
@@ -1044,7 +1086,7 @@ class UrlBase (object):
         - url_data.column: int
           Column number of this URL at parent document, or -1
         """
-        return containers.AttrDict(valid=self.valid,
+        return dict(valid=self.valid,
           extern=self.extern[0],
           cached=self.cached,
           result=self.result,
@@ -1064,8 +1106,41 @@ class UrlBase (object):
           cache_url_key=self.cache_url_key,
         )
 
+    def to_wire (self):
+        return CompactUrlData(self.to_wire_dict())
+
 
 def filter_tidy_errors (errors):
     """Filter certain errors from HTML tidy run."""
     return [x for x in errors if not \
         (x.severity=='W' and x.message=='<table> lacks "summary" attribute')]
+
+
+urlDataAttr = [
+    'valid',
+    'extern',
+    'cached',
+    'result',
+    'warnings',
+    'name',
+    'title',
+    'parent_url',
+    'base_ref',
+    'base_url',
+    'url',
+    'checktime',
+    'dltime',
+    'dlsize',
+    'info',
+    'line',
+    'column',
+    'cache_url_key',
+]
+
+class CompactUrlData (object):
+    __slots__ = urlDataAttr
+
+    def __init__(self, wired_url_data):
+        '''Set all attributes according to the dictionnary wired_url_data'''
+        for attr in urlDataAttr:
+            setattr(self, attr, wired_url_data[attr])
