@@ -20,8 +20,10 @@ Store metadata and options.
 
 import sys
 import os
+import re
 import logging.config
 import urllib
+import urlparse
 import shutil
 import _LinkChecker_configdata as configdata
 from .. import (log, LOG_CHECK, LOG_ROOT, ansicolor, lognames, clamav,
@@ -31,13 +33,14 @@ from . import confparse
 Version = configdata.version
 AppName = configdata.appname
 App = AppName+u" "+Version
-Author =  configdata.author
+Author = configdata.author
 HtmlAuthor = Author.replace(u' ', u'&nbsp;')
 Copyright = u"Copyright (C) 2000-2010 "+Author
 HtmlCopyright = u"Copyright &copy; 2000-2010 "+HtmlAuthor
 AppInfo = App+u"              "+Copyright
 HtmlAppInfo = App+u", "+HtmlCopyright
 Url = configdata.url
+SupportUrl = u"http://sourceforge.net/projects/linkchecker/support"
 Email = configdata.author_email
 UserAgent = u"%s/%s (+%s)" % (AppName, Version, Url)
 Freeware = AppName+u""" comes with ABSOLUTELY NO WARRANTY!
@@ -77,6 +80,10 @@ class Configuration (dict):
         self["interactive"] = False
         # on ftp, password is set by Pythons ftplib
         self["authentication"] = []
+        self["loginurl"] = None
+        self["loginuserfield"] = "login"
+        self["loginpasswordfield"] = "password"
+        self["loginextrafields"] = {}
         self["proxy"] = urllib.getproxies()
         self["recursionlevel"] = -1
         self["wait"] = 0
@@ -121,7 +128,7 @@ class Configuration (dict):
         }
         self['csv'] = {
             "filename": "linkchecker-out.csv",
-            'separator': ',',
+            'separator': ';',
             "quotechar": '"',
         }
         self['blacklist'] = {
@@ -167,29 +174,43 @@ class Configuration (dict):
             logging.config.fileConfig(filename)
         if handler is None:
             handler = ansicolor.ColoredStreamHandler(strm=sys.stderr)
-        logging.getLogger(LOG_ROOT).addHandler(handler)
+        self.add_loghandler(handler)
         self.set_debug(debug)
         self.status_logger = status_logger
+
+    def set_debug (self, debug):
+        """Set debugging levels for configured loggers. The argument
+        is a list of logger names to enable debug for."""
+        self.set_loglevel(debug, logging.DEBUG)
+
+    def add_loghandler (self, handler):
+        """Add log handler to root logger LOG_ROOT and set formatting."""
+        logging.getLogger(LOG_ROOT).addHandler(handler)
         if self['threads'] > 0:
             format = "%(levelname)s %(threadName)s %(message)s"
         else:
             format = "%(levelname)s %(message)s"
         handler.setFormatter(logging.Formatter(format))
 
-    def set_debug (self, debug):
-        """Set debugging levels for configured loggers. The argument
-        is a list of logger names to enable debug for."""
-        if not debug:
+    def remove_loghandler (self, handler):
+        """Remove log handler from root logger LOG_ROOT."""
+        logging.getLogger(LOG_ROOT).removeHandler(handler)
+
+    def reset_loglevel (self):
+        """Reset log level to display only warnings and errors."""
+        self.set_loglevel(['all'], logging.WARN)
+
+    def set_loglevel (self, loggers, level):
+        """Set logging levels for given loggers."""
+        if not loggers:
             return
-        # set debugging on given logger names
-        if 'all' in debug:
-            debug = lognames.keys()
+        if 'all' in loggers:
+            loggers = lognames.keys()
         # disable threading if no thread debugging
-        if "thread" not in debug:
+        if "thread" not in loggers and level == logging.DEBUG:
             self['threads'] = 0
-        for name in debug:
-            logname = lognames[name]
-            logging.getLogger(logname).setLevel(logging.DEBUG)
+        for key in loggers:
+            logging.getLogger(lognames[key]).setLevel(level)
 
     def logger_new (self, loggertype, **kwargs):
         """
@@ -222,56 +243,140 @@ class Configuration (dict):
         else:
             cfiles = files[:]
         if not cfiles:
-            # system wide config settings
-            spath = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
-            cfiles.append(spath)
-            # per user config settings
-            upath = normpath("~/.linkchecker/linkcheckerrc")
-            cfiles.append(upath)
-            copy_sys_config(spath, upath)
+            cfiles.extend(get_standard_config_files())
         # weed out invalid files
         cfiles = [f for f in cfiles if os.path.isfile(f)]
         log.debug(LOG_CHECK, "reading configuration from %s", cfiles)
         confparse.LCConfigParser(self).read(cfiles)
         self.sanitize()
 
+    def add_auth (self, user=None, password=None, pattern=None):
+        if not user or not pattern:
+            log.warn(LOG_CHECK,
+            _("warning: missing user or URL pattern in authentication data."))
+            return
+        entry = dict(
+            user=user,
+            password=password,
+            pattern=re.compile(pattern),
+        )
+        self["authentication"].append(entry)
+
+    def get_user_password (self, url):
+        """Get tuple (user, password) from configured authentication
+        that matches the given URL.
+        Both user and password can be None if not specified, or no
+        authentication matches the given URL.
+        """
+        for auth in self["authentication"]:
+            if auth['pattern'].match(url):
+                return (auth['user'], auth['password'])
+        return (None, None)
+
     def sanitize (self):
         "Make sure the configuration is consistent."
         if self["anchors"]:
-            if not self["warnings"]:
-                self["warnings"] = True
-                from ..checker import Warnings
-                self["ignorewarnings"] = Warnings.keys()
-            if 'url-anchor-not-found' in self["ignorewarnings"]:
-                self["ignorewarnings"].remove('url-anchor-not-found')
-        self['logger'] = self.logger_new(self['output'])
+            self.sanitize_anchors()
+        if self['logger'] is None:
+            self.sanitize_logger()
         if self['checkhtml']:
-            try:
-                import tidy
-            except ImportError:
-                log.warn(LOG_CHECK,
-                    _("warning: tidy module is not available; " \
-                     "download from http://utidylib.berlios.de/"))
-                self['checkhtml'] = False
+            self.sanitize_checkhtml()
         if self['checkcss']:
-            try:
-                import cssutils
-            except ImportError:
-                log.warn(LOG_CHECK,
-                    _("warning: cssutils module is not available; " \
-                     "download from http://cthedot.de/cssutils/"))
-                self['checkcss'] = False
+            self.sanitize_checkcss()
         if self['scanvirus']:
-            try:
-                clamav.init_clamav_conf(self['clamavconf'])
-            except clamav.ClamavError:
-                self['scanvirus'] = False
+            self.sanitize_scanvirus()
+        if self['storecookies']:
+            self.sanitize_cookies()
+        if self['loginurl']:
+            self.sanitize_loginurl()
+
+    def sanitize_anchors (self):
+        if not self["warnings"]:
+            self["warnings"] = True
+            from ..checker import Warnings
+            self["ignorewarnings"] = Warnings.keys()
+        if 'url-anchor-not-found' in self["ignorewarnings"]:
+            self["ignorewarnings"].remove('url-anchor-not-found')
+
+    def sanitize_logger (self):
+        if not self['output']:
+            log.warn(LOG_CHECK, _("warning: activating text logger output."))
+            self['output'] = 'text'
+        self['logger'] = self.logger_new(self['output'])
+
+    def sanitize_checkhtml (self):
+        try:
+            import tidy
+        except ImportError:
+            log.warn(LOG_CHECK,
+                _("warning: tidy module is not available; " \
+                 "download from http://utidylib.berlios.de/"))
+            self['checkhtml'] = False
+
+    def sanitize_checkcss (self):
+        try:
+            import cssutils
+        except ImportError:
+            log.warn(LOG_CHECK,
+                _("warning: cssutils module is not available; " \
+                 "download from http://cthedot.de/cssutils/"))
+            self['checkcss'] = False
+
+    def sanitize_scanvirus (self):
+        try:
+            clamav.init_clamav_conf(self['clamavconf'])
+        except clamav.ClamavError:
+            log.warn(LOG_CHECK,
+                _("warning: Clamav could not be initialized"))
+            self['scanvirus'] = False
+
+    def sanitize_cookies (self):
+        if not self['sendcookies']:
+            log.warn(LOG_CHECK, _("warning: activating sendcookies " \
+                                  "because storecookies is active."))
+            self['sendcookies'] = True
+
+    def sanitize_loginurl (self):
+        url = self["loginurl"]
+        disable = False
+        if not self["loginpasswordfield"]:
+            log.warn(LOG_CHECK,
+            _("warning: no CGI password fieldname given for login URL."))
+            disable = True
+        if not self["loginuserfield"]:
+            log.warn(LOG_CHECK,
+            _("warning: no CGI user fieldname given for login URL."))
+            disable = True
+        if self.get_user_password(url) == (None, None):
+            log.warn(LOG_CHECK,
+            _("warning: no user/password authentication data found for login URL."))
+            disable = True
+        if not url.lower().startswith(("http:", "https:")):
+            log.warn(LOG_CHECK, _("warning: login URL is not a HTTP URL."))
+            disable = True
+        urlparts = urlparse.urlsplit(url)
+        if not urlparts[0] or not urlparts[1] or not urlparts[2]:
+            log.warn(LOG_CHECK, _("warning: login URL is incomplete."))
+            disable = True
+        if disable:
+            log.warn(LOG_CHECK,
+              _("warning: disabling login URL %(url)s.") % {"url": url})
+            self["loginurl"] = None
+        elif not self['storecookies']:
+            # login URL implies storing and sending cookies
+            self['storecookies'] = self['sendcookies'] = True
 
 
-def copy_sys_config (syspath, userpath):
-    """Try to copy the system configuration to the user configuration
-    if not already done."""
+def get_standard_config_files ():
+    """Try to generate user configuration file from the system wide
+    configuration.
+    Returns tuple (system config file, user config file)."""
+    # system wide config settings
+    syspath = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
+    # per user config settings
+    userpath = normpath("~/.linkchecker/linkcheckerrc")
     if os.path.isfile(syspath) and not os.path.exists(userpath):
+        # copy the system configuration to the user configuration
         try:
             userdir = os.path.dirname(userpath)
             if not os.path.exists(userdir):
@@ -280,3 +385,4 @@ def copy_sys_config (syspath, userpath):
         except StandardError, msg:
             log.warn(LOG_CHECK, "could not copy system config from %r to %r",
                      syspath, userpath)
+    return (syspath, userpath)
