@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@ import re
 import os
 import urlparse
 import urllib
+import urllib2
+import socket
+from . import httplib2 as httplib
+from . import log, LOG_CHECK
 
 urlparse.uses_netloc.extend(('ldap', 'irc'))
 
@@ -41,9 +45,9 @@ PORT = 2
 DOCUMENT = 3
 
 default_ports = {
-    'http' : 80,
-    'https' : 443,
-    'nntps' : 563,
+    'http': 80,
+    'https': 443,
+    'nntps': 563,
 }
 
 # adapted from David Wheelers "Secure Programming for Linux and Unix HOWTO"
@@ -224,6 +228,7 @@ def url_fix_host (urlparts):
         urlparts[1] = userpass+host
     return is_idn
 
+
 def url_fix_common_typos (url):
     """Fix common typos in given URL like forgotten colon."""
     if url.startswith("http//"):
@@ -246,16 +251,15 @@ def url_parse_query (query, encoding=None):
             encoding = url_encoding
         query = query.encode(encoding, 'ignore')
     # if ? is in the query, split it off, seen at msdn.microsoft.com
-    if '?' in query:
-        query, append = query.split('?', 1)
-        append = '?'+url_parse_query(append)
-    else:
-        append = ""
+    append = ""
+    while '?' in query:
+        query, rest = query.rsplit('?', 1)
+        append = '?'+url_parse_query(rest)+append
     l = []
     for k, v, sep in parse_qsl(query, True):
         k = url_quote_part(k, '/-:,;')
         if v:
-            v = url_quote_part(v, '-:,;')
+            v = url_quote_part(v, '/-:,;')
             l.append("%s=%s%s" % (k, v, sep))
         elif v is None:
             l.append("%s%s" % (k, sep))
@@ -372,9 +376,9 @@ def url_quote (url):
     urlparts[3] = url_quote_part(urlparts[3], '&=,') # query
     l = []
     for k, v, sep in parse_qsl(urlparts[3], True): # query
-        k = url_quote_part(k, '/-:,')
+        k = url_quote_part(k, '/-:,;')
         if v:
-            v = url_quote_part(v, '/-:,')
+            v = url_quote_part(v, '/-:,;')
             l.append("%s=%s%s" % (k, v, sep))
         else:
             l.append("%s%s" % (k, sep))
@@ -475,3 +479,91 @@ def splitport (host, port=80):
         if iport:
             port = iport
     return host, port
+
+
+class PasswordManager (object):
+    """Simple password manager storing username and password. Suitable
+    for use as an AuthHandler instance in urllib2."""
+
+    def __init__ (self, user, password):
+        """Store given username and password."""
+        self.user = user
+        self.password = password
+
+    def add_password (self, realm, uri, user, passwd):
+        """Does nothing since username and password are already stored.
+
+        @return: None
+        """
+        pass
+
+    def find_user_password (self, realm, authuri):
+        """Get stored username and password.
+
+        @return: A tuple (user, password)
+        @rtype: tuple
+        """
+        return self.user, self.password
+
+
+def get_opener (user=None, password=None, proxy=None, debuglevel=0):
+    """Construct an URL opener object. It considers the given credentials
+    and proxy.
+
+    @return: URL opener
+    @rtype: urllib2.OpenerDirector
+    """
+    from . import httputil
+    pwd_manager = PasswordManager(user, password)
+    handlers = [
+        urllib2.UnknownHandler,
+        httputil.HttpWithGzipHandler(debuglevel=debuglevel),
+        urllib2.HTTPBasicAuthHandler(pwd_manager),
+        urllib2.HTTPDigestAuthHandler(pwd_manager),
+    ]
+    if proxy:
+        handlers.insert(0,
+          urllib2.ProxyHandler({"http": proxy, "https": proxy},
+                               debuglevel=debuglevel))
+        handlers.extend([
+            urllib2.ProxyBasicAuthHandler(pwd_manager),
+            urllib2.ProxyDigestAuthHandler(pwd_manager),
+        ])
+    if hasattr(httplib, 'HTTPS'):
+        handlers.append(httputil.HttpsWithGzipHandler(debuglevel=debuglevel))
+    return urllib2.build_opener(*handlers)
+
+
+def get_content (url, user=None, password=None, proxy=None, data=None):
+    """Get URL content and info.
+
+    @return: (url info, content), or (None, None) on error.
+    @rtype: tuple (string, string)
+    """
+    from . import configuration
+    if log.is_debug(LOG_CHECK):
+        debuglevel = 1
+    else:
+        debuglevel = 0
+    headers = {
+        'User-Agent': configuration.UserAgent,
+        # makes problems with some sites
+        #'Accept-Encoding': 'gzip;q=1.0, deflate;q=0.9, identity;q=0.5',
+    }
+    req = urllib2.Request(url, data, headers)
+    try:
+        f = get_opener(user=user, password=password, proxy=proxy,
+                       debuglevel=debuglevel)
+        res = None
+        try:
+            res = f.open(req)
+            return (res.info(), res.read())
+        finally:
+            if res is not None:
+                res.close()
+    except (urllib2.HTTPError, socket.timeout, urllib2.URLError,
+            socket.gaierror, socket.error, IOError, httplib.HTTPException,
+            ValueError), msg:
+        log.warn(LOG_CHECK, ("Could not get content of URL %(url)s: %(msg)s.") \
+          % {"url": url, "msg": str(msg)})
+    return (None, None)
