@@ -131,20 +131,31 @@ def has_proxy ():
         s.close()
         return True
     except StandardError:
-        pass
-    return False
+        return False
 
 need_proxy = _need_func(has_proxy, "proxy")
 
 
 @memoized
-def has_newsserver (server):
+def has_pyftpdlib ():
+    """Test if pyftpdlib is available."""
     try:
-        import nntplib
-        nntp = nntplib.NNTP(server, usenetrc=False)
-        nntp.close()
+        import pyftpdlib
         return True
-    except StandardError:
+    except ImportError:
+        return False
+
+need_pyftpdlib = _need_func(has_pyftpdlib, "pyftpdlib")
+
+
+@memoized
+def has_newsserver (server):
+    import nntplib
+    try:
+        nntp = nntplib.NNTP(server, usenetrc=False)
+        nntp.quit()
+        return True
+    except nntplib.NNTPError:
         return False
 
 
@@ -172,6 +183,7 @@ def has_pyqt ():
 
 need_pyqt = _need_func(has_pyqt, "PyQT")
 
+
 @contextmanager
 def _limit_time (seconds):
     """Raises LinkCheckerInterrupt if given number of seconds have passed."""
